@@ -62,6 +62,12 @@ void AbstractTasksModel::requestNewInstance(const QModelIndex &index)
     Q_UNUSED(index)
 }
 
+void AbstractTasksModel::requestOpenUrls(const QModelIndex &index, const QList<QUrl> &urls)
+{
+    Q_UNUSED(index)
+    Q_UNUSED(urls)
+}
+
 void AbstractTasksModel::requestClose(const QModelIndex &index)
 {
     Q_UNUSED(index)
@@ -113,6 +119,12 @@ void AbstractTasksModel::requestVirtualDesktop(const QModelIndex &index, qint32
     Q_UNUSED(desktop)
 }
 
+void AbstractTasksModel::requestActivities(const QModelIndex &index, const QStringList &activities)
+{
+    Q_UNUSED(index)
+    Q_UNUSED(activities)
+}
+
 void AbstractTasksModel::requestPublishDelegateGeometry(const QModelIndex &index, const QRect &geometry, QObject *delegate)
 {
     Q_UNUSED(index)
